import os.path

from hexagonal.domain.raw_python_file import RawPythonFile


class RawPythonFileBuilder:
    _file_full_path: str
    _file_name: str
    _file_folder: str
    _project_source_folder_full_path: str

    @property
    def file_full_path(self) -> str:
        return self._file_full_path

    def __init__(self, file_full_path: str, project_source_folder_full_path: str):
        if not file_full_path.startswith(project_source_folder_full_path):
            raise Exception('File path and project path do not match.')

        if not file_full_path.endswith('.py'):
            raise Exception('File must have .py extension.')

        self._file_full_path = os.path.normcase(os.path.normpath(file_full_path))
        self._project_source_folder_full_path = os.path.abspath(project_source_folder_full_path)

    def _get_file_name_from_full_file_path(self) -> str:
        return self._file_full_path.split(os.sep)[-1]

    def _get_file_folder_path(self) -> str:
        return os.path.abspath(self.file_full_path.replace(self._file_name, ''))

    def _get_relative_folder_path_from_project_folder(self) -> str:
        folder_relative_path = self.file_full_path.replace(self._project_source_folder_full_path, '')
        folder_relative_path = folder_relative_path.replace(self._file_name, '')
        return folder_relative_path[0:-1]

    def build(self) -> RawPythonFile:
        self._file_name = self._get_file_name_from_full_file_path()
        self._file_folder = self._get_file_folder_path()
        return RawPythonFile(
            file_full_path=self._file_full_path,
            file_name=self._file_name,
            file_folder_full_path=self._file_folder,
            relative_folder_path_from_project_folder=self._get_relative_folder_path_from_project_folder(),
            project_folder_full_path=self._project_source_folder_full_path)



from dataclasses import dataclass


@dataclass
class HexagonalError:
    message: str
    outer_layer_name: str
    inner_layer_name: str
    python_file_problem: str
    imported_module_problem: str
    group_inter_dependency: bool



import os
from typing import List

from hexagonal.domain.hexagonal_error import HexagonalError
from hexagonal.domain.hexagonal_project.hexagonal_project import HexagonalProject
from hexagonal.domain.python_file import PythonFile
from hexagonal.domain.raw_python_file import RawPythonFile
from hexagonal.hexagonal_config import HexagonalConfig
from hexagonal.services.hexagonal_dependency_flow_checker import HexagonalDependencyFlowChecker, DependencyFlowError
from hexagonal.services.hexagonal_project_builder import HexagonalProjectBuilder
from hexagonal.services.python_file_builder import PythonFileBuilder
from hexagonal.services.raw_python_project_importer import RawPythonFilesImporter


class HexagonalCheckResponse:
    errors: List[HexagonalError]
    python_files: List[PythonFile]
    hexagonal_project: HexagonalProject


class CheckProjectCoherenceUseCase:
    _source_folder_full_path: str
    _source_folder: str
    _hexagonal_config: HexagonalConfig

    def __init__(self, *, hexagonal_config: HexagonalConfig, source_folder: str = ''):
        self._source_folder = os.path.abspath(source_folder)
        self._hexagonal_config = hexagonal_config

    def check(self) -> HexagonalCheckResponse:
        python_files = self._get_python_files_from_project()
        hexagonal_project_builder = HexagonalProjectBuilder(
            python_files=python_files,
            hexagonal_composition=self._hexagonal_config.layers)
        hexagonal_project = hexagonal_project_builder.build()

        flow_checker = HexagonalDependencyFlowChecker(hexagonal_project=hexagonal_project)
        flow_checker_response = flow_checker.check()

        result = HexagonalCheckResponse()
        result.errors = self._convert_flow_errors(flow_checker_response.errors)
        result.python_files = python_files
        result.hexagonal_project = hexagonal_project
        return result

    def _get_python_files_from_project(self) -> List[PythonFile]:
        importer = RawPythonFilesImporter(source_folder_full_path=self._source_folder,
                                          hexagonal_composition=self._hexagonal_config.layers,
                                          excluded_folders=self._hexagonal_config.excluded_dirs)
        raw_python_files = importer.import_raw_python_files()
        return self._convert_raw_python_files_into_python_files(raw_python_files=raw_python_files)

    @staticmethod
    def _convert_raw_python_files_into_python_files(raw_python_files: List[RawPythonFile]) -> List[PythonFile]:
        result = []
        python_file_builder = PythonFileBuilder()
        for raw_python_file in raw_python_files:
            result.append(python_file_builder.build(raw_python_file=raw_python_file))

        return result

    @staticmethod
    def _convert_flow_errors(errors: List[DependencyFlowError]) -> List[HexagonalError]:
        result = []
        for error in errors:

            if error.group_inter_dependency:
                message = 'A file from a directory group is pointing to a file in another directory group in same layer.'
            else:
                message = 'Wrong dependency flow. An inner layer is pointing to an outer layer.'

            new_error = HexagonalError(
                message=message,
                outer_layer_name=error.imported_module_layer.name,
                inner_layer_name=error.source_file_layer.name,
                python_file_problem=error.source_file.file_full_path,
                imported_module_problem=error.imported_module.file_full_path,
                group_inter_dependency=error.group_inter_dependency
            )
            result.append(new_error)

        return result



import os
from dataclasses import dataclass
from typing import List


@dataclass
class HexagonalLayer:
    name: str
    directories_groups: List[List[str]]

    def __post_init__(self):
        self._valid_dirs()

    def _valid_dirs(self):
        for dirs in self.directories_groups:
            for dir in dirs:
                if not dir.startswith('/'):
                    raise Exception(f'Hexagonal Layer directory "{self.name}" must start with /. Example: "/domain"')

                if dir.endswith('/'):
                    raise Exception(
                        f'Hexagonal Layer directory "{self.name}" must not finish with /. Example: "/domain"')



from dataclasses import dataclass
from typing import List

from hexagonal.domain.hexagonal_project.hexagonal_project import HexagonalProject
from hexagonal.domain.hexagonal_project.hexagonal_project_layer import HexagonalProjectLayer
from hexagonal.domain.python_file import PythonFile


@dataclass
class DependencyFlowError:
    source_file: PythonFile
    source_file_layer: HexagonalProjectLayer
    imported_module: PythonFile
    imported_module_layer: HexagonalProjectLayer
    group_inter_dependency: bool


@dataclass
class DependencyFlowResponse:
    errors: List[DependencyFlowError]


class HexagonalDependencyFlowChecker:
    _hexagonal_project: HexagonalProject

    def __init__(self, hexagonal_project: HexagonalProject):
        self._hexagonal_project = hexagonal_project

    def check(self) -> DependencyFlowResponse:
        errors = []
        for layer in self._hexagonal_project.layers:
            errors.extend(self._check_layer(layer=layer))
        return DependencyFlowResponse(errors=errors)

    def _check_layer(self, *, layer: HexagonalProjectLayer) -> List[DependencyFlowError]:
        result = []
        for python_file in layer.python_files:
            result.extend(self._check_file(layer=layer, python_file=python_file))
        return result

    @staticmethod
    def _is_inner_layer_is_point_to_outer_layer(inner_layer: HexagonalProjectLayer,
                                                outer_layer: HexagonalProjectLayer) -> bool:
        return inner_layer.index < outer_layer.index

    @staticmethod
    def _are_files_in_same_layer(source_layer: HexagonalProjectLayer,
                                 module_layer: HexagonalProjectLayer) -> bool:
        return source_layer.index == module_layer.index

    @staticmethod
    def _is_source_file_pointer_to_file_in_different_dir_group(source_file: PythonFile,
                                                               module: PythonFile,
                                                               layer: HexagonalProjectLayer) -> bool:
        source_file_dir_id = layer.get_directory_group_index_for_file(
            relative_folder_path_from_project_folder=source_file.relative_folder_path_from_project_folder)
        module_dir_id = layer.get_directory_group_index_for_file(
            relative_folder_path_from_project_folder=module.relative_folder_path_from_project_folder
        )

        return source_file_dir_id != module_dir_id

    def _check_file(self, *, layer: HexagonalProjectLayer, python_file: PythonFile) -> List[DependencyFlowError]:
        result = []
        for imported_module_path in python_file.imported_modules:
            module_layer = self._hexagonal_project.get_layer_for_file_path(file_full_path=imported_module_path)
            if not module_layer:
                continue

            imported_module = self._hexagonal_project.get_python_file(file_full_path=imported_module_path)

            if self._is_inner_layer_is_point_to_outer_layer(inner_layer=layer, outer_layer=module_layer):
                error = DependencyFlowError(
                    source_file=python_file,
                    source_file_layer=layer,
                    imported_module=imported_module,
                    imported_module_layer=module_layer,
                    group_inter_dependency=False)
                result.append(error)
                continue

            if self._are_files_in_same_layer(source_layer=layer, module_layer=module_layer):
                if self._is_source_file_pointer_to_file_in_different_dir_group(source_file=python_file,
                                                                               module=imported_module,
                                                                               layer=layer):
                    error = DependencyFlowError(
                        source_file=python_file,
                        source_file_layer=layer,
                        imported_module=imported_module,
                        imported_module_layer=module_layer,
                        group_inter_dependency=True)
                    result.append(error)

        return result



from typing import List

from hexagonal.domain.hexagonal_layer import HexagonalLayer
from hexagonal.services.hexagonal_composition import HexagonalComposition


class HexagonalConfig:
    excluded_dirs: List[str]
    _layers: HexagonalComposition

    def __init__(self):
        self._layers = HexagonalComposition()
        self.excluded_dirs = []

    def add_inner_layer(self, layer: HexagonalLayer) -> 'HexagonalConfig':
        self._layers.append(layer)
        return self

    def add_inner_layer_with_dirs(self, layer_name: str, directories: List[str]) -> 'HexagonalConfig':
        self._layers.append(HexagonalLayer(name=layer_name, directories_groups=[directories]))
        return self

    def add_inner_layer_with_dirs_groups(self, layer_name: str,
                                         directories_groups: List[List[str]]) -> 'HexagonalConfig':
        self._layers.append(HexagonalLayer(name=layer_name, directories_groups=directories_groups))
        return self

    def clear_layers(self):
        self._layers.clear()

    @property
    def layers(self) -> HexagonalComposition:
        return self._layers


hexagonal_config = HexagonalConfig()



from typing import Optional, List

from hexagonal.domain.hexagonal_layer import HexagonalLayer


class HexagonalComposition(List[HexagonalLayer]):

    def __add__(self, next_layer) -> 'HexagonalComposition':
        self.clear()
        self.append(next_layer)
        return self

    def __rshift__(self, next_layer: HexagonalLayer) -> 'HexagonalComposition':
        self.append(next_layer)
        return self



import logging

from hexagonal.domain.python_file import PythonFile
from hexagonal.domain.raw_python_file import RawPythonFile
from hexagonal.services.python_file_imports_resolver import PythonFileImportsResolver


class PythonFileBuilder:

    @staticmethod
    def build(raw_python_file: RawPythonFile) -> PythonFile:
        try:
            imports_resolver = PythonFileImportsResolver(raw_python_file=raw_python_file)
            imported_modules = imports_resolver.resolve_imported_modules()
        except Exception as error:
            logging.error(f'Error while loading file imports: {raw_python_file.file_full_path}', exc_info=error)
            raise error

        return PythonFile(
            file_full_path=raw_python_file.file_full_path,
            file_name=raw_python_file.file_name,
            file_folder_full_path=raw_python_file.file_folder_full_path,
            relative_folder_path_from_project_folder=raw_python_file.relative_folder_path_from_project_folder,
            project_folder_full_path=raw_python_file.project_folder_full_path,
            imported_modules=imported_modules)



from typing import List

from diagrams import Diagram, Cluster
from diagrams.aws.storage import Storage

from hexagonal.hexagonal_config import HexagonalConfig


class GenerateDiagramUseCase:
    _PADDING_CHARS = 17

    def execute(self, *, project_name: str, hexagonal_config: HexagonalConfig, show: bool = False,
                output_file_name: str = '') -> bool:
        last_node = None

        with Diagram(project_name, show=show, filename=output_file_name):
            for layer in hexagonal_config.layers:
                with Cluster(self._get_adjusted_name_for_layer(cur_name=layer.name, groups=layer.directories_groups)):
                    is_first = True
                    if len(layer.directories_groups) == 1:
                        for dirs in layer.directories_groups:
                            for module in dirs:
                                cur_node = Storage(label=module)
                                if is_first:
                                    is_first = False
                                    if last_node is not None:
                                        last_node >> cur_node

                                last_node = cur_node
                    else:
                        for idx, dirs in enumerate(layer.directories_groups):
                            with Cluster(self._get_adjusted_name_for_cluster(cur_name=str(idx + 1), dirs=dirs)):
                                for module in dirs:
                                    cur_node = Storage(label=module)
                                    if is_first:
                                        is_first = False
                                        if last_node is not None:
                                            last_node >> cur_node

                                    last_node = cur_node

        return last_node is not None

    def _get_adjusted_name_for_layer(self, cur_name: str, groups: List[List[str]]) -> str:
        max_length = 0
        new_name = cur_name
        for group in groups:
            new_name = self._get_adjusted_name_for_cluster(cur_name=cur_name, dirs=group)
            if len(new_name) > max_length:
                max_length = len(new_name)

        return new_name

    def _get_adjusted_name_for_cluster(self, cur_name: str, dirs: List[str]) -> str:
        max_length = len(cur_name)
        for dir_name in dirs:
            if len(dir_name) > max_length:
                max_length = len(dir_name)

        max_length -= len(cur_name)
        return cur_name + ' ' * (max_length + self._PADDING_CHARS)



import os.path
from dataclasses import dataclass


@dataclass
class RawPythonFile:
    file_full_path: str
    file_name: str
    file_folder_full_path: str
    # For instance if the project is located at: /usr/project/
    # and the file is /usr/project/services/another/file.py
    # the project_relative_folder_path is "services/another"
    relative_folder_path_from_project_folder: str
    project_folder_full_path: str

    def __post_init__(self):
        self.file_full_path = os.path.normcase(os.path.normpath(self.file_full_path))
        self.file_folder_full_path = os.path.normcase(os.path.normpath(self.file_folder_full_path))
        self.relative_folder_path_from_project_folder = os.path.normcase(
            os.path.normpath(self.relative_folder_path_from_project_folder))
        self.project_folder_full_path = os.path.normcase(os.path.normpath(self.project_folder_full_path))



import os
from dataclasses import dataclass
from typing import List, Optional

from hexagonal.domain.python_file import PythonFile


@dataclass
class HexagonalProjectLayer:
    index: int
    name: str
    directories_groups: List[List[str]]
    python_files: List[PythonFile]

    def get_python_file(self, file_full_path: str) -> Optional[PythonFile]:
        for python_file in self.python_files:
            if python_file.file_full_path == file_full_path:
                return python_file
        return None

    def get_directory_group_index_for_file(self, relative_folder_path_from_project_folder: str) -> int:
        if len(self.directories_groups) == 1:
            return 0

        for dir_group_index, dir_group in enumerate(self.directories_groups):
            for dir_name in dir_group:
                if os.path.normcase(os.path.normpath(dir_name)) == relative_folder_path_from_project_folder:
                    return dir_group_index + 1

        return 0

    def is_file_in_layer(self, file_full_path: str) -> bool:
        if self.get_python_file(file_full_path=file_full_path):
            return True
        return False



import logging
import os
import sys
from runpy import run_path

import click

from hexagonal.domain.hexagonal_error import HexagonalError
from hexagonal.domain.hexagonal_layer import HexagonalLayer
from hexagonal.hexagonal_config import hexagonal_config
from hexagonal.services.toml_configuration import TomlConfiguration
from hexagonal.use_cases.check_project_coherence_usecase import CheckProjectCoherenceUseCase, HexagonalCheckResponse
from hexagonal.use_cases.generate_diagram_usecase import GenerateDiagramUseCase


@click.group(help='Options for Hexagonal Coherence Check')
def cli():
    pass


@click.command()
@click.option('--project_path', help='Main project folder.', required=True)
@click.option('--source_path', help='Where main source folder is located.', required=True)
@click.option('--hexagonal_config_file', default='hexagonal_config.py', help="Hexagonal configuration file's name.")
@click.option('--show', default=True, help="Show diagram file upon creation.")
def diagram(project_path, source_path, hexagonal_config_file, show):
    try:
        _process_cli_arguments(project_path=project_path,
                               source_path=source_path,
                               hexagonal_config_file=hexagonal_config_file)

        hexa_diagram = GenerateDiagramUseCase()
        hexa_diagram.execute(project_name='Hexagonal Architecture Diagram', hexagonal_config=hexagonal_config,
                             show=show)
    except Exception as error:
        click.echo(f'Error while generating project\'s diagram: "{error}"')
        exit(1)


@click.command()
@click.option('--project_path', help='Main project folder.', required=True)
@click.option('--source_path', help='Where main source folder is located.', required=True)
@click.option('--hexagonal_config_file', default='hexagonal_config.py', help="Hexagonal configuration file's name.")
def check(project_path: str, source_path: str, hexagonal_config_file: str):
    def _build_response_message() -> str:
        return f'Hexagonal Architecture: Checked a project with {len(response.hexagonal_project.layers)} ' \
               f'hexagonal layers, {len(response.python_files)} python files ' \
               f'and found {len(response.errors)} errors.'

    try:
        _process_cli_arguments(project_path=project_path,
                               source_path=source_path,
                               hexagonal_config_file=hexagonal_config_file)

        checker = CheckProjectCoherenceUseCase(hexagonal_config=hexagonal_config, source_folder=source_path)
        response = checker.check()
        _print_check_response(response)
    except Exception as error:
        logging.error('Error while processing project', exc_info=error)
        click.echo(f'Error while processing project: "{error}"')
        exit(1)

    [_print_error(index, error) for index, error in enumerate(response.errors)]
    if len(response.errors) > 0:
        logging.error(_build_response_message())
        exit(1)

    click.echo(_build_response_message())


def _print_check_response(response: HexagonalCheckResponse):
    hexa_project = response.hexagonal_project
    logging.info(f'Checked information for project: {hexa_project.project_path}')

    for layer in hexa_project.layers:
        logging.info(f'#### Files in layer "{layer.name}"')

        for idx, file in enumerate(layer.python_files):
            logging.info(f' {idx + 1}) File {file.file_full_path}')

    logging.warning(f'#### Files out site of layers')
    for idx, file in enumerate(hexa_project.files_not_in_layers):
        logging.warning(f' {idx + 1}) File {file.file_full_path}')


def _print_error(error_index: int, error: HexagonalError):
    if error_index == 0:
        click.echo('')
        click.echo('#' * 100)
        click.echo('## Hexagonal Architecture errors found:')
        click.echo('## First fix the [ERROR 1] and re-run the check. '
                   'Follows errors can be cascading from it. \n')

    if error.group_inter_dependency:
        click.echo(f'[ERROR {error_index + 1}] Hexagonal Architecture: {error.message}')
        click.echo(f'    Cross modules dependency in layer: {error.inner_layer_name}')
        click.echo(f'    Python file: {error.python_file_problem}')
        click.echo(f'    Outer Module : {error.imported_module_problem}')
        return

    click.echo(f'[ERROR {error_index + 1}] Hexagonal Architecture: {error.message}')
    click.echo(f'    Wrong flow: {error.inner_layer_name} -> {error.outer_layer_name}')
    click.echo(f'    Python file: {error.python_file_problem}')
    click.echo(f'    Outer Module : {error.imported_module_problem}')


def _validate_project_source_path(source_path: str):
    click.echo(f'Checking project at: {source_path}')
    if not os.path.isdir(source_path):
        click.echo('Project folder not found.')
        exit(1)
    sys.path.append(source_path)


def _process_cli_arguments(*, project_path: str, source_path: str, hexagonal_config_file: str):
    source_path = os.path.abspath(source_path)

    _validate_project_source_path(source_path=source_path)
    _process_configuration(project_path=project_path,
                           source_path=source_path,
                           hexagonal_config_file=hexagonal_config_file)


def _process_configuration(project_path: str, source_path: str, hexagonal_config_file: str):
    hexagonal_config.clear_layers()
    hexagonal_config.excluded_dirs = []

    if _import_configuration_from_toml_file(project_path=project_path):
        return False

    hexagonal_config_file = source_path + '/' + hexagonal_config_file
    _import_hexagonal_config_file(hexagonal_config_file=hexagonal_config_file)


def _import_configuration_from_toml_file(project_path: str) -> bool:
    toml_file = project_path + '/pyproject.toml'
    if not os.path.isfile(toml_file):
        return False

    config = TomlConfiguration()
    config.load_from_file(file_path=toml_file)

    if not config.has_hexagonalpy_configuration():
        return False

    click.echo(f'Loading configuration from {toml_file}')
    hexagonal_config.excluded_dirs = config.excluded_dirs()

    for layer in config.layers():
        hexagonal_config.add_inner_layer(HexagonalLayer(name=layer['name'],
                                                        directories_groups=layer['directories_groups']))

    return True


def _import_hexagonal_config_file(hexagonal_config_file: str):
    click.echo(f'Checking hexagonal configuration file at: {hexagonal_config_file}')
    if not os.path.isfile(hexagonal_config_file):
        click.echo('Project configuration file not found.')
        exit(1)
    hexagonal_config.clear_layers()
    run_path(hexagonal_config_file)


cli.add_command(check)
cli.add_command(diagram)


def main():
    logging.basicConfig(format='%(asctime)s %(levelname)s: %(message)s', stream=sys.stdout, level=logging.INFO)
    cli()


if __name__ == '__main__':
    main()



import os.path
from modulefinder import ModuleFinder, Module
from typing import List, Optional

from hexagonal.domain.raw_python_file import RawPythonFile


class PythonFileImportsResolver:
    _raw_python_file: RawPythonFile

    def __init__(self, raw_python_file: RawPythonFile):
        self._raw_python_file = raw_python_file

        if not os.path.isfile(raw_python_file.file_full_path):
            raise Exception(f'Source file not found: {raw_python_file.file_full_path}')

        self._raw_python_file = raw_python_file

    def resolve_imported_modules(self) -> List[str]:
        all_modules = self._get_all_modules_source_paths()

        if self._raw_python_file.file_full_path in all_modules:
            all_modules.remove(self._raw_python_file.file_full_path)

        return all_modules

    def _get_all_modules_source_paths(self) -> List[str]:
        # finder = ModuleFinder(path=[self._raw_python_file.project_folder_full_path])
        finder = ModuleFinder(path=[])
        finder.run_script(self._raw_python_file.file_full_path)

        all_modules = []
        for module in finder.modules.values():
            all_modules.append(self.__get_module_file_name(module))

        all_modules.extend(
            self._convert_bad_modules_into_paths(bad_modules=list(finder.badmodules.keys())))

        return [os.path.normcase(os.path.normpath(value)) for value in all_modules if value is not None]

    @staticmethod
    def __get_module_file_name(module: Module) -> Optional[str]:
        string_representation = str(module).strip()
        if not string_representation:
            return None

        try:
            return str(module).split("'")[3]
        except IndexError:
            return None
        except Exception:
            return None

    def _convert_bad_modules_into_paths(self, *, bad_modules: List[str]) -> List[str]:
        result = []

        for bad_module in bad_modules:
            bad_module_file = bad_module.replace('.', os.sep) + '.py'

            while os.sep in bad_module_file:
                full_bad_module_file = self._raw_python_file.project_folder_full_path + os.sep + bad_module_file

                if os.path.isfile(full_bad_module_file):
                    result.append(os.path.normpath(full_bad_module_file))
                    break

                bad_module_file = bad_module_file.split(os.sep, 1)[1]

        return result



from dataclasses import dataclass
from typing import List


@dataclass
class PythonFile:
    file_full_path: str
    file_name: str
    file_folder_full_path: str
    # For instance if the project is located at: /usr/project/
    # and the file is /usr/project/services/another/file.py
    # the project_relative_folder_path is "services/another"
    relative_folder_path_from_project_folder: str
    project_folder_full_path: str
    imported_modules: List[str]



import os
from glob import glob
from typing import List

from hexagonal.domain.raw_python_file import RawPythonFile
from hexagonal.services.hexagonal_composition import HexagonalComposition
from hexagonal.services.raw_python_file_builder import RawPythonFileBuilder


class RawPythonFilesImporter:
    _composition: HexagonalComposition
    _source_folder_full_path: str
    _excluded_folders: List[str]

    @property
    def source_folder_full_path(self):
        return self._source_folder_full_path

    def __init__(self, source_folder_full_path: str, hexagonal_composition: HexagonalComposition,
                 excluded_folders: List[str]):
        if not os.path.isabs(source_folder_full_path):
            raise Exception("The param source_folder_full_path must have the source's folder full path.")

        if not os.path.isdir(source_folder_full_path):
            raise Exception('Source folder not found.')

        if not excluded_folders:
            self._excluded_folders = []
        else:
            self._excluded_folders = excluded_folders

        self._source_folder_full_path = source_folder_full_path
        self._composition = hexagonal_composition

    def import_raw_python_files(self) -> List[RawPythonFile]:
        project_files_paths = self._get_all_python_files_paths_from_source_folder()
        python_project_files = self._convert_files_paths_in_python_project_files(python_files_paths=project_files_paths)

        return python_project_files

    def _get_all_python_files_paths_from_source_folder(self) -> List[str]:
        all_files = [os.path.abspath(y) for x in os.walk(self._source_folder_full_path)
                     for y in glob(os.path.join(x[0], '*.py'))]
        result = []
        for file in all_files:
            include_file = True

            if f'{os.sep}.' in file:
                continue

            for excluded_dir in self._excluded_folders:
                file_relative_path = file.replace(self._source_folder_full_path, '')
                if file_relative_path.startswith(excluded_dir):
                    include_file = False
                    break

            if include_file:
                result.append(file)

        return result

    def _convert_files_paths_in_python_project_files(self, python_files_paths: List[str]) -> List[RawPythonFile]:
        result = []
        for python_files_path in python_files_paths:
            result.append(self._convert_file_path_in_python_project_file(python_file_path=python_files_path))

        return result

    def _convert_file_path_in_python_project_file(self, python_file_path: str) -> RawPythonFile:
        builder = RawPythonFileBuilder(file_full_path=python_file_path,
                                       project_source_folder_full_path=self._source_folder_full_path)
        return builder.build()



from dataclasses import dataclass
from typing import List, Optional

from hexagonal.domain.hexagonal_project.hexagonal_project_layer import HexagonalProjectLayer
from hexagonal.domain.python_file import PythonFile


@dataclass
class HexagonalProject:
    project_path: str
    layers: List[HexagonalProjectLayer]
    files_not_in_layers: List[PythonFile]

    def get_layer_for_file_path(self, file_full_path: str) -> Optional[HexagonalProjectLayer]:
        for layer in self.layers:
            if layer.is_file_in_layer(file_full_path=file_full_path):
                return layer
        return None

    def get_python_file(self, file_full_path: str) -> PythonFile:
        for layer in self.layers:
            python_file = layer.get_python_file(file_full_path=file_full_path)
            if python_file:
                return python_file

        raise Exception(f'File not found {file_full_path}')



from typing import List

from hexagonal.domain.hexagonal_project.hexagonal_project import HexagonalProject
from hexagonal.domain.hexagonal_project.hexagonal_project_layer import HexagonalProjectLayer
from hexagonal.domain.python_file import PythonFile
from hexagonal.services.hexagonal_composition import HexagonalComposition


class HexagonalProjectBuilder:
    _python_files: List[PythonFile]
    _project_folder_full_path: str

    def __init__(self, python_files: List[PythonFile], hexagonal_composition: HexagonalComposition):
        self._python_files = python_files
        self._hexagonal_composition = hexagonal_composition

    def build(self) -> HexagonalProject:
        if not self._python_files:
            raise Exception('No python files found.')

        self._project_folder_full_path = self._extract_project_path()
        project_layers = self._generate_layers()
        self._add_files_to_corresponding_layers(project_layers=project_layers)
        files_not_in_layers = self._get_files_without_layers(project_layers=project_layers)

        return HexagonalProject(project_path=self._project_folder_full_path,
                                layers=project_layers,
                                files_not_in_layers=files_not_in_layers)

    def _extract_project_path(self) -> str:
        return self._python_files[0].project_folder_full_path

    def _generate_layers(self) -> List[HexagonalProjectLayer]:
        result = []
        for idx, composition_layer in enumerate(reversed(self._hexagonal_composition)):
            project_layer = HexagonalProjectLayer(
                index=idx + 1,
                name=composition_layer.name,
                directories_groups=composition_layer.directories_groups,
                python_files=[]
            )
            result.append(project_layer)

        result.sort(key=lambda item: item.index)
        return result

    def _add_files_to_corresponding_layers(self, project_layers: List[HexagonalProjectLayer]):
        for python_file in self._python_files:
            for layer in project_layers:
                relative_path = python_file.relative_folder_path_from_project_folder
                for dirs in layer.directories_groups:
                    if any(relative_path.startswith(layer_dir) for layer_dir in dirs):
                        layer.python_files.append(python_file)
                        break

    def _get_files_without_layers(self, project_layers: List[HexagonalProjectLayer]) -> List[PythonFile]:
        result = []
        for python_file in self._python_files:
            file_add = False
            for layer in project_layers:
                relative_path = python_file.relative_folder_path_from_project_folder
                for dirs in layer.directories_groups:
                    if any(relative_path.startswith(layer_dir) for layer_dir in dirs):
                        file_add = True
                        break

            if not file_add:
                result.append(python_file)

        return result



from typing import MutableMapping, Any, List

import toml


class TomlConfigurationException(Exception):
    pass


class TomlConfiguration:
    _configuration: MutableMapping[str, Any]

    def load_from_file(self, file_path: str):
        with open(file_path) as f:
            content = f.read()
            self.load_from_string(content=content)

    def load_from_string(self, *, content: str) -> MutableMapping[str, Any]:
        self._configuration = toml.loads(content)
        return self._configuration

    def has_hexagonalpy_configuration(self) -> bool:
        try:
            self._configuration['tool']['hexagonalpy']
        except KeyError:
            return False

        return True

    def excluded_dirs(self) -> List[str]:
        try:
            return self._configuration['tool']['hexagonalpy']['excluded_dirs']
        except KeyError:
            return []

    def layers(self) -> List[Any]:
        try:
            result = []
            sorted_keys = list(self._configuration['tool']['hexagonalpy']['layer'].keys())
            sorted_keys.sort(reverse=True)

            for key in sorted_keys:
                result.append(self._configuration['tool']['hexagonalpy']['layer'][f'{key}'])

            return result
        except KeyError:
            return []



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['hexagonal',
 'hexagonal.domain',
 'hexagonal.domain.hexagonal_project',
 'hexagonal.infrastructure',
 'hexagonal.services',
 'hexagonal.use_cases']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0',
 'diagrams>=0.20.0,<0.21.0',
 'toml>=0.10.2,<0.11.0',
 'types-toml>=0.10.3,<0.11.0']

entry_points = \
{'console_scripts': ['hexagonal = hexagonal.infrastructure.cli:main']}

setup_kwargs = {
    'name': 'hexagonal-py',
    'version': '0.1.5',
    'description': 'Hexagonal Coherence Check',
    'long_description': "# Hexagonal Coherence Check\n\nThis project checks if the dependency flow between the layers of the Hexagonal architecture defined \nfor this project was respected.\n\n### How to install\n\nIt can be easily installed via pip: `pip install hexagonal-py`\n\n### How to configure your project\n\nThere are two ways to configure `hexagonal-py`:\n1. Using `pyproject.toml` (recommended)\n2. Using `hexagonal_config.py`, which is expected to be on your main source folder. \n\nIt's necessary to define your hexagonal layers and their order.\nGiven for example, the project structure below:\n```\npyproject.toml (Optinal)\n. src\n├── __init__.py\n├── hexagonal_config.py (Optional)\n├── domain\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 ├── __pycache__\n│\xa0\xa0 └── person.py\n├── infrastructure\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 └── person_mysql_repository.py\n├── main.py\n├── services\n│\xa0\xa0 ├── __init__.py\n│\xa0\xa0 └── person_repository.py\n└── usecases\n    ├── __init__.py\n    └── create_person_usecase.py\n.tests    \n```\nGeneral aspects:\n1. Existing layers: `domain`, `infrastructure`, `services`, `usecases`.\n2. Which should respect the following dependency flow: `infrastructure` -> `usecases` -> `services` -> `domain`\n3. Exclude `tests` from checks\n\nIf you are using `pyproject.toml`, you would have this:\n```toml\n[tool.hexagonalpy]\nexcluded_dirs = ['/tests']\n\n[tool.hexagonalpy.layer.1]\nname = 'Domain'\ndirectories_groups = [['/domain']]\n\n[tool.hexagonalpy.layer.2]\nname = 'Services'\ndirectories_groups = [['/services']]\n\n[tool.hexagonalpy.layer.3]\nname = 'Use Cases'\ndirectories_groups = [['/usecases']]\n\n[tool.hexagonalpy.layer.4]\nname = 'Infrastructure'\ndirectories_groups = [['/infrastructure']]\n```\n\nIf you are using `hexagonal_config.py`:\n```python\nfrom hexagonal.hexagonal_config import hexagonal_config\n\nhexagonal_config.add_inner_layer_with_dirs(layer_name='infrastructure', directories=['/infrastructure'])\nhexagonal_config.add_inner_layer_with_dirs(layer_name='use_cases', directories=['/use_cases'])\nhexagonal_config.add_inner_layer_with_dirs(layer_name='services', directories=['/services'])\nhexagonal_config.add_inner_layer_with_dirs(layer_name='domain', directories=['/domain'])\n\nhexagonal_config.excluded_dirs = ['/tests']\n```\n\n#### Extra content\n\n1. excluded_dirs  \nList of directories that you want to exclude from the `hexagonal-py` validation.  \nSyntax: `excluded_dirs = ['/tests', '/another_folder', '/another_folder2']`\n\n\n2. Layers  \nList of layers you defined in your project. \nThere are 3 aspects you need to fill in for a layer: `layer order`, `name`, `directories_groups`.\n\n2.1. Layer order: The number of the layers tells the order of the dependency flow between them. \nWhere the most inner layer is `1` and the most outer layer is the greater number. Example:\n\n```toml\n[tool.hexagonalpy.layer.1] # Layer 1, as it's the most inner layer, and it can't point to any other layer but all the \n                           # other layers can point to it.\nname = 'domain'\ndirectories_groups = [['/domain']]\n```\n\n2.2. Name: The readable name of the layer, that will be used for documentation, internal messages etc.\n\n2.3. Directories_groups: It's a list of a list. You can specify which folders belong to the given layer, and you can also \ndefine that some folders can't point to other folders inside the same layer. For instance, the `MySql` and `Postgres` \ncomponents belongs to `Infrastructure Layer` but **can't** refer to each other.\n\n```toml\n[tool.hexagonalpy.layer.4] \nname = 'Infrastructure'\ndirectories_groups = [['/Infrastructure/MySql'],['Infrastructure/Postgres']]\n```\n\n### Generating the Project Diagram\nThis command generate a visual diagram show the composition of your hexagonal layers.\n\n#### Pre requisites\nTo generate the Hexagonal Diagram of the project, it's necessary to have Graphviz installed in the machine.  \nFor Mac you can ``brew install graphviz``.  \nFor other, check the documentation https://graphviz.org/download/. \n\n#### CMD\n`hexagonal diagram --project_path ./ --source_path ./src` \n\n### Checking Project's Hexagonal Integrity \nThis checks if the correct flow of the dependencies -from outer to inner layer- was respected.\n\n#### CMD\n`hexagonal check --project_path ./ --source_path ./src`\n\n",
    'author': 'rfrezino',
    'author_email': 'rodrigofrezino@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
